local addonName, farmerVars = ...

local chipId = 129100
local specialList = {
  [116158] = true, --Lunarfall Carp
  [112633] = true, -- Frostdeep Minnow
  [127141] = true, -- Bloated Thresher
  [32620] = true, --time-lost scroll
  [86547] = true, --skyshard
  [25433] = true, --obsidian warbeads
  [141975] = true, --mark of aquaos
  [138777] = true, --drowned mana
  [87779] = true, --Ancient Guo-Lai Cache Key
}

local font = CreateFont('farmerFont')
local _, chipName = GetItemInfoInstant(chipId)
local playerName = UnitName('player')
local messagePatterns = {
  ['CHAT_MSG_LOOT'] = {
    {
      LOOT_ITEM_SELF_MULTIPLE, {
        {
          '%%s',
          '(.+)'
        }, {
          '(%%d)',
          '(%%d+)'
        }
      }
    },
    {
      LOOT_ITEM_SELF, {
        {
          '%%s',
          '(.+)',
        }
      }
    },
    {
      LOOT_ITEM_PUSHED_SELF_MULTIPLE,
      {
        {
          '%%s',
          '(.+)'
        },
        {
          '(%%d)',
          '(%%d+)'
        }
      }
    },
    {
      LOOT_ITEM_PUSHED_SELF, {
        {
          '%%s',
          '(.+)',
        }
      }
    }
  },
  ['CHAT_MSG_CURRENCY'] = {
    {
      CURRENCY_GAINED_MULTIPLE, {
        {
          '%%s',
          '(.+)'
        }, {
          '(%%d)',
          '(%%d+)'
        }
      }
    },
    {
      CURRENCY_GAINED, {
        {
          '%%s',
          '(.+)',
        }
      }
    }
  }
}

local farmerFrame
local events = {}
local itemList = {}
local awaitingList = {}
local mapShown
local bagUpdate
local hadChip = false
local lootStack = nil

for msg, replacements in pairs(messagePatterns) do
  local new = {}
  for index, content in pairs(replacements) do
    local message = content[1]
    local patterns = content[2]
    for _, pattern in pairs(patterns) do
      message = gsub(message, pattern[1], pattern[2])
    end
    message = '^' .. message .. '$'
    new[index] = message
  end
  messagePatterns[msg] = new
end

local function printTable (table)
  for i, v in pairs(table) do
    print(i, ' - ', v)
  end
end

local function setTrueScale (frame, scale)
    frame:SetScale(1)
    frame:SetScale(scale / frame:GetEffectiveScale())
end

local function performAutoLoot ()
  -- for i = 1, GetNumLootItems(), 1 do
  for i = GetNumLootItems(), 1, -1 do
    LootSlot(i)
  end
end

local function printItem (texture, text, colors)
  local icon = ' |T' .. texture .. ':0|t '

  farmerFrame:AddMessage(icon .. text, unpack(colors))
end

local function printItemCount (texture, text, count, colors)
  if (count <= 1) then
    printItem(texture, text, colors)
  else
    printItem(texture, text .. ' x' .. count, colors)
  end
end

local function printStackableItem (id, texture, count, colors)
  local totalCount
  local text

  if (bagUpdate == true) then
    totalCount = GetItemCount(id, true)
  else
    totalCount = count + GetItemCount(id, true)
  end

  text = 'x' .. count .. ' (' .. totalCount .. ')'

  printItem(texture, text, colors)
end

local function getTooltipAP (itemLink)
  local text = ''
  local pattern

  GameTooltip:SetOwner(UIParent,'ANCHOR_NONE')
  GameTooltip:SetHyperlink(itemLink)

  pattern = _G['GameTooltipTextLeft2']:GetText()
  pattern = string.sub(pattern, 11)
  pattern = string.sub(pattern, 1, -3)
  pattern = '(%d+.*)' .. pattern

  for i=3, GameTooltip:NumLines(), 1 do
    local line = _G['GameTooltipTextLeft' .. i]:GetText() or ''
    text = text .. line
  end

  text = text:match(pattern)
  if (text ~= nil) then
    return text .. 'AP'
  end

  return 'AP'
end

local function handleItem (itemLink, count)
  local itemName, _itemLink, itemRarity, itemLevel, itemMinLevel, itemType,
        itemSubType, itemStackCount, itemEquipLoc, texture,
        itemSellPrice, itemClassID, itemSubClassID, bindType, expacID,
        itemSetID, isCraftingReagent = GetItemInfo(itemLink)
  -- happens when caging a pet
  if (itemName == nil) then
    return
  end

  local itemId = GetItemInfoInstant(itemLink)

  if (farmerOptions.reagents == true) then
    if (isCraftingReagent == true) then
      if (itemId == chipId and hadChip == true) then
        hadChip = false
        return
      end

      printStackableItem(itemLink, texture, count, {0, 0.8, 0.8, 1})
      return
    end

    if (itemId ~= nil) then
      if ((itemId >= 130200 and
           itemId <= 130204) or
          itemId == 129099) then
        hadChip = true
        printStackableItem(chipId, texture, count, {1, 1, 1, 1})
        return
      end
    end
  end

  if ((farmerOptions.rarity == true and
       itemRarity >= farmerOptions.minimumRarity) or
      (farmerOptions.special == true and
       itemId ~= nil and
       specialList[itemId] == true)) then
    local colors = farmerVars.rarityColors[itemRarity]

    if (itemStackCount > 1) then
      printStackableItem(itemLink, texture, count, colors)
    elseif (itemClassID == 1) then --bags
      printItemCount(texture, itemType, count, colors)
    elseif (itemClassID == 3 and
            itemSubClassID == 11) then
      local text
      itemLevel = GetDetailedItemLevelInfo(itemLink)
      text = itemSubType .. ' ' .. itemLevel
      printItemCount(texture, text, count, colors)
    elseif (itemEquipLoc ~= '') then
      itemLevel = GetDetailedItemLevelInfo(itemLink)
      printItemCount(texture, itemLevel, count, colors)
    elseif (IsArtifactPowerItem(itemLink) == true) then
      local text = getTooltipAP(itemLink)
      printItemCount(texture, text, count, colors)
    else
      printItemCount(texture, itemName, count, colors)
    end
    return
  end
end

function handleCurrency (link, count)
  local name, amount, texture, earnedThisWeek, weeklyMax, totalMax, isDicovered,
        rarity = GetCurrencyInfo(link)
  local text = amount .. ' (' .. count .. ')'

  printItem(texture, text, {1, 0.9, 0, 1})
end

function displayLoot ()
  if (lootStack == nil) then
    return
  end

  for key, value in pairs (lootStack) do
    handleItem(key, lootStack[key])
  end

  lootStack = nil
end

--[[
///#############################################################################
/// Event listeners
///#############################################################################
]]--

function events:LOOT_READY (lootSwitch)
  mapShown = WorldMapFrame:IsShown()
  if (lootSwitch == 1 and
      farmerOptions.fastLoot == true) then
    performAutoLoot()
    if (LootFrame:IsShown() == true) then
      LootFrame:SetAlpha(1)
    end
  else
    LootFrame:SetAlpha(1)
  end
end

function events:LOOT_CLOSED ()
  LootFrame:SetAlpha(0)
  if (mapShown == true and
      WorldMapFrame:IsShown() == false) then
    ToggleFrame(WorldMapFrame)
    mapShown = false
  end
end

function events:CHAT_MSG_LOOT (message, _, _, _, unit)
  local t = debugprofilestop()
  -- prevents string parsing in groups/raids
  if (unit ~= playerName) then
    return
  end

  local list = messagePatterns['CHAT_MSG_LOOT']

  if (lootStack == nil) then
    lootStack = {}
    C_Timer.After(0, displayLoot)
  end

  for k, v in pairs(list) do
    local link, amount = string.match(message, v)
    if link ~= nil then
      if (amount == nil) then
        amount = 1
      end

      if (lootStack[link] == nil) then
        lootStack[link] = amount
      else
        lootStack[link] = lootStack[link] + amount
      end
      return
    end
  end
end

function events:BAG_UPDATE_DELAYED ()
  if (lootStack ~= nil) then
    bagUpdate = true
    displayLoot()
    bagUpdate = false
  end
end

function events:CHAT_MSG_CURRENCY (message)
  if (farmerOptions.currency ~= true) then
    return
  end

  local list = messagePatterns['CHAT_MSG_CURRENCY']

  for k, v in pairs(list) do
    local link, amount = string.match(message, v)
    if (link ~= nil) then
      if (amount == nil) then
        amount = 1
      end

      handleCurrency(link, amount)
      break
    end
  end
end

function events:PLAYER_MONEY ()
  if (farmerOptions.money == false) then
    return
  end

  local money = GetMoney()

  if (farmerVars.moneyStamp >= money) then
    farmerVars.moneyStamp = money
    return
  end

  local difference = money - farmerVars.moneyStamp;
  local gold = math.floor(difference / 10000)
  local silver = math.floor(difference / 100) % 100
  local copper = difference % 100
  local text = ''

  farmerVars.moneyStamp = money

  if (gold ~= 0) then
    gold = GOLD_AMOUNT_TEXTURE:format(gold, 0, 0)
    text = text .. ' ' .. gold
  end
  if (silver ~= 0) then
    silver = SILVER_AMOUNT_TEXTURE:format(silver, 0, 0)
    text = text .. ' ' .. silver
  end
  if (copper ~= 0) then
    copper = COPPER_AMOUNT_TEXTURE:format(copper, 0, 0)
    text = text .. ' ' .. copper
  end
  text = string.sub(text, 2)
  farmerFrame:AddMessage(text, 1, 1, 1, 1)
end

farmerFrame = CreateFrame('ScrollingMessageFrame', 'farmerFrame', UIParent)
farmerFrame:SetWidth(GetScreenWidth() / 2)
farmerFrame:SetHeight(GetScreenHeight() / 2)
farmerFrame:SetSpacing(10)
farmerFrame:SetFrameStrata('DIALOG')
farmerFrame:SetFrameLevel(2)
farmerFrame:SetFading(true)
farmerFrame:SetTimeVisible(3)
farmerFrame:SetFadeDuration(1)
farmerFrame:SetMaxLines(20)
farmerFrame:SetInsertMode('TOP')
farmerFrame:SetFontObject(font)
setTrueScale(farmerFrame, 1)
farmerFrame:Show()

local function eventHandler (self, event, ...)
  events[event](self, ...)
end

farmerFrame:SetScript('OnEvent', eventHandler)

for k, v in pairs(events) do
  farmerFrame:RegisterEvent(k)
end

--[[
///#############################################################################
/// shared variables
///#############################################################################
--]]

farmerVars.frame = farmerFrame
farmerVars.font = font
